package com.flyer.foster.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.flyer.exception.BusinessException;
import com.flyer.foster.dto.*;
import com.flyer.foster.entity.Area;
import com.flyer.foster.entity.Garden;
import com.flyer.foster.mapper.IGardenMapper;
import com.flyer.foster.service.IAreaService;
import com.flyer.foster.service.IGardenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 果园 服务实现类
 * </p>
 *
 * @author flyer
 * @since 2024-05-09
 */
@Service
public class GardenServiceImpl extends ServiceImpl<IGardenMapper, Garden> implements IGardenService {
    @Autowired
    private IAreaService iAreaService;

    @Override
    public IPage<GardenRespDTO> getPageList(IPage<GardenRespDTO> page, GardenQueryDTO queryDTO) {
        IPage<GardenRespDTO> pageList = baseMapper.getPageList(page, queryDTO);
        for (GardenRespDTO record : pageList.getRecords()) {
            // 获取区域
            List<String> areaNameList = iAreaService.lambdaQuery()
                    .eq(Area::getGardenId, record.getId())
                    .list()
                    .stream().map(Area::getName).collect(Collectors.toList());
            record.setAreaNameList(areaNameList);
        }
        return pageList;
    }

    @Transactional
    @Override
    public boolean addGarden(GardenAddDTO addDTO) {
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        Garden garden = new Garden();
        garden.setTenantId(tenantId);
        BeanUtil.copyProperties(addDTO, garden);
        return this.save(garden);
    }

    @Transactional
    @Override
    public boolean deleteByIdList(List<Integer> idList) {
        return baseMapper.deleteByIdList(idList);
    }

    @Transactional
    @Override
    public boolean updateGarden(GardenUpdateDTO updateDTO) {
        Garden garden = baseMapper.selectById(updateDTO.getId());
        if (garden == null) {
            throw new BusinessException("无效的果园id");
        }
        return this.lambdaUpdate()
                .set(Garden::getName, updateDTO.getName())
                .set(updateDTO.getUserId() != null, Garden::getUserId, updateDTO.getUserId())
                .set(StrUtil.isNotBlank(updateDTO.getBaseMapUrl()), Garden::getBaseMapUrl, updateDTO.getBaseMapUrl())
                .set(updateDTO.getStatus() != null, Garden::getStatus, updateDTO.getStatus())
                .eq(Garden::getId, updateDTO.getId())
                .update();
    }
}
